import Enmap from "enmap";
import { Client, GuildMember } from "discord.js";

import { ModeratorEmitter } from "./ModeratorEmitter";

import { MuteManager } from "./managers/MuteManager";
import { WarnManager } from "./managers/WarnManager";
import { FetchManager } from "./managers/FetchManager";
import { UtilsManager } from "./managers/UtilsManager";

import { ClearData, DefaultObject } from "./ModeratorData";
import { ModeratorGuildData } from "./ModeratorDatabase";

import { ClearType, Events, MuteType, PunishmentType } from "./ModeratorEnums";

import { ModeratorOptions } from "./ModeratorOptions";
import { BanOptions, ClearOptions, KickOptions, Options, PunishOptions, UnbanOptions } from "./Options";

export declare class Moderator extends ModeratorEmitter {
    constructor(client: Client, options?: ModeratorOptions);

    private client: Client;

    public database: Enmap<string, ModeratorGuildData>;
    
    public author: string;
    public website: string;
    public version: string;

    public ready: boolean;
    public readyTimestamp: number;
    public options: ModeratorOptions;

    public mutes: MuteManager;
    public warns: WarnManager;
    public fetcher: FetchManager;
    public utils: UtilsManager;

    /**
     * Method for kicking Discord guild users
     * 
     * @param options Kick options
     * 
     * @returns Returns the state of the action or an object with an error
     */
    public kick(options: Options<KickOptions>): Promise<DefaultObject>;

    /**
     * Method for banning Discord guild users
     * 
     * @param options Ban options
     * 
     * @returns Returns the state of the action or an object with an error
     */
    public ban(options: Options<BanOptions>): Promise<DefaultObject>;

    /**
     * Method for removing bans from Discord guild users
     * 
     * @param options Unban options
     * 
     * @returns Returns the state of the action or an object with an error
     */
    public unban(options: Options<UnbanOptions>): Promise<DefaultObject>;

    /**
     * Method for punishing Discord guild users
     * 
     * @param options Punish options
     * 
     * @returns Returns the state of the action or an object with an error
     */
    public punish(options: Options<PunishOptions>): Promise<DefaultObject>;

    /**
     * Method for deleting messages in text channels
     * 
     * @param options Clear options
     * 
     * @returns Returns the number of deleted messages or an object with an error
     */
    public clear(options: Options<ClearOptions>): Promise<ClearData & DefaultObject>;

    /**
     * Method for module initialization
     * 
     * @param options Moderator Options
     */
    private init(options?: ModeratorOptions): void;

    /**
     * Method for initializing the module database
     */
    private initDatabase(): void;

    /**
     * Method for initializing all module managers
     */
    private initManagers(): void;
}

export { ClearType, Events, MuteType, PunishmentType }