import { GuildMember } from "discord.js";

import { Moderator } from "../Moderator";
import { GuildWarn } from "../ModeratorDatabase";

import { ClearOptions, CreationWarnOptions, DefaultObject } from "../ModeratorData";

import { Options, FetchOptions, ListOptions, CreateWarnOptions, DeleteWarnOptions } from "../Options";

export declare class WarnManager {
    constructor(moderator: Moderator);

    private moderator: Moderator;

    /**
     * Method for issuing a warning to a Discord guild user
     * 
     * @param options Create warn options
     * 
     * @returns Returns warning data or an object with an error
     */
    public add(options: Options<CreateWarnOptions>): Promise<GuildWarn & DefaultObject>;

    /**
     * Method for getting warnings data
     * 
     * @param options Fetch options
     * 
     * @returns Returns information about a warning or an object with an error
     */
    public get(options: Options<FetchOptions>): Promise<GuildWarn & DefaultObject>;

    /**
     * A method to get a list of warnings for a guild or user
     * 
     * @param options Fetch options
     * 
     * @returns Returns a list of warnings
     */
    public list(options: Options<ListOptions>): Promise<Array<GuildWarn>>;

    /**
     * A method for removing warnings from a user in the Discord guild
     * 
     * @param options Delete warn options
     * 
     * @returns Returns altered warning data or an object with an error
     */
    public delete(options: Options<DeleteWarnOptions>): Promise<GuildWarn & DefaultObject>;
}